﻿-- addon name and localization table

local a,L=...

-- create frames

local DCL=CreateFrame('frame',a)
local DCLP=CreateFrame('frame',a..'Panel')
local DCLU=CreateFrame('frame',a..'Update')
DCL:RegisterEvent('PLAYER_LOGIN')

-- initialize variables

Decliner_SV={}
local _
local DCLD={}
local DCLD_OnHide
local DCLD_OnShow
local DCL_Filter
local DCL_SetPetitionFrame
local DCL_SetStaticPopup
local DCL_SetTradeFrame
local g='|cff00ff00'
local msglist={}
local msgtimer
local namerealm
local notemp=true
local numbox=0
local numtxt=0
local oldSFX
local plrlist={}
local plrtimer
local r='|cffff0000'
local selfwarn
local soundhandle
local SPD_={}
local temp={}
local tradeinitiated
local update
local vars={}
local defs={
	duel={enabled=true,tell=false,guild=false,friend=false,bnet=false,msg=false,},
	pet={enabled=true,tell=false,guild=false,friend=false,bnet=false,msg=false,},
	guild={enabled=true,tell=false,friend=false,bnet=false,msg=false,},
	group={enabled=true,tell=false,guild=false,friend=false,bnet=false,msg=false,},
	chart={enabled=true,tell=false,guild=false,friend=false,bnet=false,msg=false,},
	trade={enabled=true,tell=false,guild=false,friend=false,bnet=false,msg=false,},
	dcl={msgs=false,disable=false,},
}

-- global string strip
-- originally from Phanx

function DCL:Pattern(str)
	str=gsub(str,'|H.*|h.*|h','') -- removes player links
	str=gsub(str,'%%%d?$?d','') -- removes %d
	str=gsub(str,'%%%d?$?s','') -- removes %s
	str=gsub(str,':.*','') -- removes colons and anything after them
	str=gsub(str,'%(','') -- removes (
	str=gsub(str,'%)','') -- removes )
	str=gsub(str,'%.','') -- removes periods
	str=gsub(str,'^%s','') -- removes leftover space from beginning of string
	str=gsub(str,'%s$','') -- removes leftover space from end of string
	return str
end

-- Blizzard global strings used, with enUS in comments. Parts in quotes are raw strings before DCL:Pattern.
-- this table is only here to keep strings in one place. it's inefficient to do this, especially since some of these are used once.

local S={
	BNF=BATTLENET_OPTIONS_LABEL..' '..FRIENDS,                     --  Battle.net Friends
	CAN=CANCEL,                                                    --  Cancel
	DCA='.*'..DCL:Pattern(ERR_DUEL_CANCELLED)..'.*',               -- "Duel cancelled."
	DDE=DISABLE..' '..a,                                           --  Disable Decliner
	DDM=DISABLE..' '..strupper(ALL)..' '..a..' '..SYSTEM_MESSAGES, --  Disable ALL Decliner System Messages
	DRE='.*'..DCL:Pattern(DUEL_REQUESTED)..'.*',                   -- "%s has challenged you to a duel."
	DUE=DUEL,                                                      --  Duel
	EDE=ENABLE..' '..DECLINE,                                      --  Enable Decline
	GLD=GUILD,                                                     --  Guild
	GME=DCL:Pattern(INSPECT_GUILD_NUM_MEMBERS),                    -- "%d Guild Members"
	GRI='.*'..DCL:Pattern(ERR_INVITED_TO_GROUP_SS)..'.*',          -- "|Hplayer:%s|h[%s]|h has invited you to join a group."
	GRO=GROUP,                                                     --  Group
	GUI='.*'..DCL:Pattern(ERR_INVITED_TO_GUILD_SSS)..'.*',         -- "|Hplayer:%s|h[%s]|h invites you to join %s."
	HID=HIDE,                                                      --  Hide
	INV=DCL:Pattern(GUILDEVENT_TYPE_INVITE),                       -- "%s invites %s"
	OKA=OKAY,                                                      --  Okay
	PBA=PET_BATTLE_PVP_QUEUE,                                      --  Pet Battle
	PBD='.*'..DCL:Pattern(ERR_PETBATTLE_DECLINED)..'.*',           -- "Pet battle invitation declined."
	PBR='.*'..DCL:Pattern(PET_BATTLE_PVP_DUEL_REQUESTED)..'.*',    -- "%s has challenged you to a pet-battle."
	PTN=DCL:Pattern(PETITION_TITLE),                               -- "Petition: %s"
	REA=strlower(READY),                                           --  Ready
	REQ=GUILDINFOTAB_APPLICANTS_NONE,                              --  Requests
	RFR=DCL:Pattern(FRIENDS_LIST_REALM)..' '..FRIENDS,             -- "Realm: %s" Friends
	SME=SYSTEM_MESSAGES,                                           --  System Messages
	TCA='.*'..DCL:Pattern(ERR_TRADE_CANCELLED)..'.*',              -- "Trade cancelled."
	TIN='.*'..DCL:Pattern(ERR_INITIATE_TRADE_S)..'.*',             -- "You have requested to trade with %s."
	TRA=TRADE,                                                     --  Trade
	VER=strlower(GAME_VERSION_LABEL),                              --  Version
	WHI=DCL:Pattern(AUTOCOMPLETE_LABEL_INTERACTED),                -- "(Whispered)"
}

-- timer to control chat messages and list of whispered players

function DCL:Timer(str)
	if str == 'msg' then
		msgtimer=true
		C_Timer.After(60,function()
			tremove(msglist,1)
			if next(msglist) ~= nil then
				DCL:Timer('msg')
			else
				msgtimer=false
			end
		end)
	elseif str == 'plr' then
		plrtimer=true
		C_Timer.After(60,function()
			tremove(plrlist,1)
			if next(plrlist) ~= nil then
				DCL:Timer('plr')
			else
				plrtimer=false
			end
		end)
	end
end

-- chat output
-- the idea for faking a yellow system message came from Jaliborc's Scrap
-- but they hook into the MessageHandler function while I use AddMessage

function DCL:Print(msg)
	if vars.dcl.msgs then return end
	if next(msglist) ~= nil then
		for k,v in next,msglist do
			if msg == v then return end
		end
	end
	if not selfwarn then tinsert(msglist,msg) end selfwarn=false
	if not msgtimer then DCL:Timer('msg') end
	local info,notfound=ChatTypeInfo['SYSTEM'],true
	for i=1,NUM_CHAT_WINDOWS do
		local chatframe=_G['ChatFrame'..i]
		if chatframe:IsEventRegistered('CHAT_MSG_SYSTEM') then
			notfound=false
			chatframe:AddMessage(r..a..': |r'..msg,info.r,info.g,info.b,info.id)
		end
	end
	if notfound then
		DEFAULT_CHAT_FRAME:AddMessage(r..a..': |r'..msg,info.r,info.g,info.b,info.id)
	end
end

-- main bouncer, which ultimately decides if a player is autodeclined

function DCL:IsAllowed(toon,var)
	namerealm=toon
	if vars.dcl.disable or not vars[var].enabled then return true end
	if next(plrlist) ~= nil then
		for k,v in next,plrlist do
			if not vars[var].tell and namerealm == v then
			return true end
		end
	end
	if var ~= 'guild' then
		if not vars[var].guild and IsInGuild() then
			for i=1,(GetNumGuildMembers()) do
				if namerealm == Ambiguate((GetGuildRosterInfo(i)),'mail') then
				return true end
			end
		end
	end
	if not vars[var].friend then
		for i=1,(GetNumFriends()) do
			if namerealm == Ambiguate((GetFriendInfo(i)),'mail') then
			return true end
		end
	end
	if not vars[var].bnet then
		for i=1,(BNGetNumFriends()) do
			local _,_,_,_,_,_,client,online=BNGetFriendInfo(i)
			if online and client == 'WoW' then
				for j=1,(BNGetNumFriendToons(i)) do
					local _,name,_,realm=BNGetFriendToonInfo(i,j)
					if namerealm == Ambiguate(name..'-'..realm,'mail') then
					return true end
				end
			end
		end
	end
	return false
end

-- following function from http://lua-users.org/wiki/CopyTable
-- this copies the values of a table into another table instead of creating a reference

local function tcopy(src)
	local copy
	if type(src) == 'table' then
		copy={}
		for k,v in next,src do
			copy[tcopy(k)]=tcopy(v)
		end
		setmetatable(copy,tcopy(getmetatable(src)))
	else
		copy=src
	end
	return copy
end

-- begin event blocks

function DCL:PLAYER_LOGIN()
	GuildRoster()
	ShowFriends()
	oldSFX=GetCVar('Sound_EnableSFX')
	SPD_['DUEL_REQUESTED']=StaticPopupDialogs['DUEL_REQUESTED']
	SPD_['PET_BATTLE_PVP_DUEL_REQUESTED']=StaticPopupDialogs['PET_BATTLE_PVP_DUEL_REQUESTED']
	SPD_['PARTY_INVITE']=StaticPopupDialogs['PARTY_INVITE']
	SPD_['PARTY_INVITE_XREALM']=StaticPopupDialogs['PARTY_INVITE_XREALM']
	if Decliner_SV.saved then
		for k1,v1 in next,defs do
			if Decliner_SV[k1] == nil then
				Decliner_SV[k1]=defs[k1]
			end
			for k2,v2 in next,v1 do
				if Decliner_SV[k1][k2] == nil then
					Decliner_SV[k1][k2]=defs[k1][k2]
				elseif type(Decliner_SV[k1][k2]) ~= 'boolean' then
					Decliner_SV[k1][k2]=defs[k1][k2]
				end
			end
		end
		vars=tcopy(Decliner_SV)
		Decliner_SV.saved=true
	else
		vars=tcopy(defs)
		Decliner_SV=tcopy(defs)
		Decliner_SV.saved=true
	end
	DCL:BlizzCode()
	DCL:BTBGI()
	DCL:Options()
	DCL:EventRegister()
	DCL:Print(S['VER']..' '..GetAddOnMetadata(a,'Version')..' '..S['REA'])
	DCL:UnregisterEvent('PLAYER_LOGIN')
end

function DCL:CHAT_MSG_SYSTEM(msg)
	if strmatch(msg,S['TIN']) then tradeinitiated=true end
end

function DCL:CHAT_MSG_WHISPER_INFORM(_,name)
	tinsert(plrlist,Ambiguate(name,'mail'))
	if not plrtimer then DCL:Timer('plr') end
end

function DCL:CVAR_UPDATE(cvar,value)
	oldSFX=GetCVar('Sound_EnableSFX')
	if cvar == 'BLOCK_TRADES' then
		if value == '0' and not vars.trade.enabled then
			vars.trade.enabled=true
			DCL:EventRegister()
			DCL:SetCheckBoxes()
			DCL:BTBGI()
			selfwarn=true
			DCL:Print(L['X_BLOCK_TRADES_OFF'])
		end
		if value == '1' and vars.trade.enabled then
			vars.trade.enabled=false
			DCL:EventRegister()
			DCL:SetCheckBoxes()
			DCL:BTBGI()
			selfwarn=true
			DCL:Print(L['X_BLOCK_TRADES_ON'])
		end
	end
end

function DCL:DISABLE_DECLINE_GUILD_INVITE()
	if vars.guild.enabled then return end
	vars.guild.enabled=true
	DCL:EventRegister()
	DCL:SetCheckBoxes()
	DCL:BTBGI()
	selfwarn=true
	DCL:Print(L['X_BLOCK_GUILDS_OFF'])
end

function DCL:ENABLE_DECLINE_GUILD_INVITE()
	if not vars.guild.enabled then return end
	vars.guild.enabled=false
	DCL:EventRegister()
	DCL:SetCheckBoxes()
	DCL:BTBGI()
	selfwarn=true
	DCL:Print(L['X_BLOCK_GUILDS_ON'])
end

function DCL:DUEL_REQUESTED(name)
	local _,frame=StaticPopup_Visible('DUEL_REQUESTED')
	HideUIPanel(frame)
	namerealm=Ambiguate(name,'mail')
	if DCL:IsAllowed(namerealm,'duel') then
		DCL_SetStaticPopup(frame,'default')
		ShowUIPanel(frame)
		PlaySoundKitID(880)
	else
		CancelDuel()
		name=Ambiguate(name,'none')
		if not vars.duel.msg then
			DCL:Print(format(L['DECLINED_DUEL'],'|Hplayer:'..name..'|h['..name..']|h'))
		end
	end
end

function DCL:PET_BATTLE_PVP_DUEL_REQUESTED(name)
	local _,frame=StaticPopup_Visible('PET_BATTLE_PVP_DUEL_REQUESTED')
	HideUIPanel(frame)
	namerealm=Ambiguate(name,'mail')
	if DCL:IsAllowed(namerealm,'pet') then
		DCL_SetStaticPopup(frame,'default')
		ShowUIPanel(frame)
		PlaySoundKitID(880)
	else
		C_PetBattles.CancelPVPDuel()
		name=Ambiguate(name,'none')
		if not vars.pet.msg then
			DCL:Print(format(L['DECLINED_PET'],'|Hplayer:'..name..'|h['..name..']|h'))
		end
	end
end

function DCL:GUILD_INVITE_REQUEST(name,guild)
	SetCVar('Sound_EnableSFX','0')
	namerealm=Ambiguate(name,'mail')
	if DCL:IsAllowed(namerealm,'guild') then
		C_Timer.After(0.5,function()
			SetCVar('Sound_EnableSFX',oldSFX)
			PlaySoundKidID(888)
		end)
	else
		StaticPopupSpecial_Hide(GuildInviteFrame)
		DeclineGuild()
		name=Ambiguate(name,'none')
		if not vars.guild.msg then
			if GetLocale() == 'koKR' or GetLocale() == 'zhCN' or GetLocale() == 'zhTW' then
				DCL:Print(format(L['DECLINED_GUILD'],guild,'|Hplayer:'..name..'|h['..name..']|h'))
			else
				DCL:Print(format(L['DECLINED_GUILD'],'|Hplayer:'..name..'|h['..name..']|h',guild))
			end
		end
		C_Timer.After(0.5,function()SetCVar('Sound_EnableSFX',oldSFX)end)
	end
end

function DCL:PARTY_INVITE_REQUEST(name)
	local _,frame,which=StaticPopup_Visible('PARTY_INVITE'),'PARTY_INVITE'
	if not frame then
		local _,frame,which=StaticPopup_Visible('PARTY_INVITE_XREALM'),'PARTY_INVITE_XREALM'
	end
	HideUIPanel(frame)
	namerealm=Ambiguate(name,'mail')
	if DCL:IsAllowed(namerealm,'group') then
		DCL_SetStaticPopup(frame,'default')
		ShowUIPanel(frame)
		PlaySoundKitID(880)
		StaticPopupDialogs[which].OnHide=function(self)
			if not self.inviteAccepted then
				DeclineGroup()
				self:Hide()
			end
			StaticPopupDialogs[which]=DCLD[which]
		end
	else
		DeclineGroup()
		name=Ambiguate(name,'none')
		if not vars.group.msg then
			DCL:Print(format(L['DECLINED_GROUP'],'|Hplayer:'..name..'|h['..name..']|h'))
		end
	end
end

function DCL:PETITION_SHOW()
	HideUIPanel(PetitionFrame)
	local ptype,_,_,_,name,originator=GetPetitionInfo()
	namerealm=Ambiguate(name,'mail')
	if originator or DCL:IsAllowed(namerealm,'chart') then
		DCL_SetPetitionFrame('default')
		ShowUIPanel(PetitionFrame)
	else
		ClosePetition()
		name=Ambiguate(name,'none')
		if not vars.chart.msg then
			if ptype == 'guild' then
				DCL:Print(format(L['DECLINED_CHART_GUILD'],'|Hplayer:'..name..'|h['..name..']|h'))
			else
				DCL:Print(format(L['DECLINED_CHART_NIL'],'|Hplayer:'..name..'|h['..name..']|h'))
			end
		end
	end
end

function DCL:TRADE_SHOW()
	HideUIPanel(TradeFrame)
	local name,realm=UnitName('npc') -- thank you to http://wowprogramming.com/docs/api_types#unitID for that vague Trade UI note next to 'npc'
	if realm then namerealm=Ambiguate(name..'-'..realm,'mail')
	else namerealm=Ambiguate(name,'mail') end
	if tradeinitiated or DCL:IsAllowed(namerealm,'trade') then
		DCL_SetTradeFrame('default')
		ShowUIPanel(TradeFrame)
	else
		CloseTrade()
		name=Ambiguate(name,'none')
		if not vars.trade.msg then
			DCL:Print(format(L['DECLINED_TRADE'],'|Hplayer:'..name..'|h['..name..']|h'))
		end
	end
end

function DCL:TRADE_CLOSED()
	tradeinitiated=false
end

function DCL:UI_INFO_MESSAGE(msg)
	if strmatch(msg,S['DCA']) and not DCL:IsAllowed(namerealm,'duel') then UIErrorsFrame:Clear() end
	if strmatch(msg,S['PBD']) and not DCL:IsAllowed(namerealm,'pet') then UIErrorsFrame:Clear() end
	if strmatch(msg,S['TCA']) and not DCL:IsAllowed(namerealm,'trade') then UIErrorsFrame:Clear() end
end

-- magic line of code that calls event blocks when their namesakes trigger
DCL:SetScript('OnEvent',function(self,event,...)self[event](self,...)end)

-- end event blocks

function DCL:BlizzCode()

-- chat frame filter that blocks associated system messages if that action is declined

	DCL_Filter=function(_,_,msg)
		if vars.dcl.disable then return false end
		if strmatch(msg,S['DRE']) and not DCL:IsAllowed(namerealm,'duel') then return true end
		if strmatch(msg,S['PBR']) and not DCL:IsAllowed(namerealm,'pet') then return true end
		if strmatch(msg,S['GUI']) and not DCL:IsAllowed(namerealm,'guild') then return true end
		if strmatch(msg,S['GRI']) and not DCL:IsAllowed(namerealm,'group') then return true end
		return false
	end
	ChatFrame_AddMessageEventFilter('CHAT_MSG_SYSTEM',DCL_Filter)

-- begin Blizzard code replacements (these are injected into the UI, so direct usage of global strings instead of S table)
-- all of these are basically alterations of Blizzard's Lua or changing Blizzard's frame scripts
-- you can view Blizzard's Lua with these instructions: http://wow.gamepedia.com/Viewing_Blizzard's_interface_code

	DCLD['DUEL_REQUESTED']={
		text=DUEL_REQUESTED,
		button1=ACCEPT,
		button2=DECLINE,
		OnAccept=function(self)
			AcceptDuel()
		end,
		OnCancel=function(self)
			CancelDuel()
		end,
		timeout=STATICPOPUP_TIMEOUT,
		hideOnEscape=1
	}

	DCLD['PET_BATTLE_PVP_DUEL_REQUESTED']={
		text=PET_BATTLE_PVP_DUEL_REQUESTED,
		button1=ACCEPT,
		button2=DECLINE,
		OnAccept=function(self)
			C_PetBattles.AcceptPVPDuel()
		end,
		OnCancel=function(self)
			C_PetBattles.CancelPVPDuel()
		end,
		timeout=STATICPOPUP_TIMEOUT,
		hideOnEscape=1
	}

	DCLD['PARTY_INVITE']={
		text=INVITATION,
		button1=ACCEPT,
		button2=DECLINE,
		OnShow=function(self)
			self.inviteAccepted=nil
		end,
		OnAccept=function(self)
			AcceptGroup()
			self.inviteAccepted=1
		end,
		OnCancel=function(self)
			DeclineGroup()
		end,
		OnHide=function(self)
			if (not self.inviteAccepted) then
				self:Hide()
			end
		end,
		timeout=STATICPOPUP_TIMEOUT,
		whileDead=1,
		hideOnEscape=1
	}

	DCLD['PARTY_INVITE_XREALM']={
		text=INVITATION_XREALM,
		button1=ACCEPT,
		button2=DECLINE,
		OnShow=function(self)
			self.inviteAccepted=nil
		end,
		OnAccept=function(self)
			AcceptGroup()
			self.inviteAccepted=1
		end,
		OnCancel=function(self)
			DeclineGroup()
		end,
		OnHide=function(self)
			if (not self.inviteAccepted) then
				self:Hide()
			end
		end,
		timeout=STATICPOPUP_TIMEOUT,
		whileDead=1,
		hideOnEscape=1
	}

	DCLD_OnShow=function(frame)
		local dialog=frame.which
		if dialog and
		((dialog == 'DUEL_REQUESTED' and vars.duel.enabled) or
		(dialog == 'PET_BATTLE_PVP_DUEL_REQUESTED' and vars.pet.enabled) or
		((dialog == 'PARTY_INVITE' or dialog == 'PARTY_INVITE_XREALM') and vars.group.enabled)) then
			local OnShow=dialog.OnShow
			if (OnShow) then
				OnShow(frame,frame.data)
			end
			if (dialog.enterClicksFirstButton) then
				frame:SetScript('OnKeyDown',StaticPopup_OnKeyDown)
			end
		else
			StaticPopup_OnShow(frame)
		end
	end

	DCLD_OnHide=function(frame)
		local dialog=frame.which
		if dialog and
		((dialog == 'DUEL_REQUESTED' and vars.duel.enabled) or
		(dialog == 'PET_BATTLE_PVP_DUEL_REQUESTED' and vars.pet.enabled) or
		((dialog == 'PARTY_INVITE' or dialog == 'PARTY_INVITE_XREALM') and vars.group.enabled)) then
			StaticPopup_CollapseTable()
			local OnHide=dialog.OnHide
			if (OnHide) then
				OnHide(frame,frame.data)
			end
			if (dialog.enterClicksFirstButton) then
				frame:SetScript('OnKeyDown',nil)
			end
		else
			StaticPopup_OnHide(frame)
		end
	end

	DCL_SetStaticPopup=function(frame,action)
		frame:SetScript('OnShow',nil)
		frame:SetScript('OnHide',nil)
		if action == 'decline' then
			frame:SetScript('OnShow',function()
				DCLD_OnShow(frame)
			end)
			frame:SetScript('OnHide',function()
				DCLD_OnHide(frame)
			end)
		elseif action == 'default' and vars.dcl.disable then
			frame:SetScript('OnShow',function()
				StaticPopup_OnShow(frame)
			end)
			frame:SetScript('OnHide',function()
				StaticPopup_OnHide(frame)
			end)
		elseif action == 'default' then
			frame:SetScript('OnShow',function()
				StaticPopup_OnShow(frame)
			end)
			frame:SetScript('OnHide',function()
				StaticPopup_OnHide(frame)
				if vars.duel.enabled or vars.pet.enabled or vars.group.enabled then
					DCL_SetStaticPopup(frame,'decline')
				end
			end)
		end
	end

	DCL_SetPetitionFrame=function(action)
		PetitionFrame:SetScript('OnShow',nil)
		PetitionFrame:SetScript('OnHide',nil)
		if action == 'decline' then
			PetitionFrame:SetScript('OnHide',function()
				StaticPopup_Hide('CONFIRM_GUILD_CHARTER_SIGNATURE')
			end)
		elseif action == 'default' and vars.dcl.disable then
			PetitionFrame:SetScript('OnShow',function()
				PlaySoundKitID(875)
			end)
			PetitionFrame:SetScript('OnHide',function()
				PlaySoundKitID(876)
				StaticPopup_Hide('CONFIRM_GUILD_CHARTER_SIGNATURE')
				ClosePetition()
			end)
		elseif action == 'default' then
			PetitionFrame:SetScript('OnShow',function()
				PlaySoundKitID(875)
			end)
			PetitionFrame:SetScript('OnHide',function()
				PlaySoundKitID(876)
				StaticPopup_Hide('CONFIRM_GUILD_CHARTER_SIGNATURE')
				ClosePetition()
				if vars.chart.enabled then DCL_SetPetitionFrame('decline') end
			end)
		end
	end

	DCL_SetTradeFrame=function(action)
		TradeFrame:SetScript('OnShow',nil)
		TradeFrame:SetScript('OnHide',nil)
		if action == 'decline' then
			TradeFrame:SetScript('OnShow',function()
				TradeFrame_OnShow(self)
			end)
			TradeFrame:SetScript('OnHide',function()
				MoneyInputFrame_SetCopper(TradePlayerInputMoneyFrame,0)
			end)
		elseif action == 'default' and vars.dcl.disable then
			TradeFrame:SetScript('OnShow',function()
				TradeFrame_OnShow(self)
				PlaySoundKitID(839)
			end)
			TradeFrame:SetScript('OnHide',function()
				TradeFrame_OnHide()
				PlaySoundKitID(840)
			end)
		elseif action == 'default' then
			TradeFrame:SetScript('OnShow',function()
				TradeFrame_OnShow(self)
				PlaySoundKitID(839)
			end)
			TradeFrame:SetScript('OnHide',function()
				TradeFrame_OnHide()
				PlaySoundKitID(840)
				if vars.trade.enabled then DCL_SetTradeFrame('decline') end
			end)
		end
	end

end

-- end Blizzard code replacements

-- main event registration based on what user options are enabled or disabled

function DCL:EventRegister()
	if vars.dcl.disable then
		DCL:UnregisterAllEvents()
		StaticPopupDialogs['DUEL_REQUESTED']=SPD_['DUEL_REQUESTED']
		StaticPopupDialogs['PET_BATTLE_PVP_DUEL_REQUESTED']=SPD_['PET_BATTLE_PVP_DUEL_REQUESTED']
		StaticPopupDialogs['PARTY_INVITE']=SPD_['PARTY_INVITE']
		StaticPopupDialogs['PARTY_INVITE_XREALM']=SPD_['PARTY_INVITE_XREALM']
		DCL_SetPetitionFrame('default')
		DCL_SetTradeFrame('default')
		for i=1,STATICPOPUP_NUMDIALOGS do
			local frame=_G['StaticPopup'..i]
			DCL_SetStaticPopup(frame,'default')
		end
	else
		DCL:RegisterEvent('CHAT_MSG_SYSTEM')
		DCL:RegisterEvent('CHAT_MSG_WHISPER_INFORM')
		DCL:RegisterEvent('CVAR_UPDATE')
		DCL:RegisterEvent('DISABLE_DECLINE_GUILD_INVITE')
		DCL:RegisterEvent('ENABLE_DECLINE_GUILD_INVITE')
		if vars.duel.enabled then
			DCL:RegisterEvent('DUEL_REQUESTED')
			StaticPopupDialogs['DUEL_REQUESTED']=DCLD['DUEL_REQUESTED']
		else
			DCL:UnregisterEvent('DUEL_REQUESTED')
			StaticPopupDialogs['DUEL_REQUESTED']=SPD_['DUEL_REQUESTED']
		end
		if vars.pet.enabled then
			DCL:RegisterEvent('PET_BATTLE_PVP_DUEL_REQUESTED')
			StaticPopupDialogs['PET_BATTLE_PVP_DUEL_REQUESTED']=DCLD['PET_BATTLE_PVP_DUEL_REQUESTED']
		else
			DCL:UnregisterEvent('PET_BATTLE_PVP_DUEL_REQUESTED')
			StaticPopupDialogs['PET_BATTLE_PVP_DUEL_REQUESTED']=SPD_['PET_BATTLE_PVP_DUEL_REQUESTED']
		end
		if vars.guild.enabled then
			DCL:RegisterEvent('GUILD_INVITE_REQUEST')
		else
			DCL:UnregisterEvent('GUILD_INVITE_REQUEST')
		end
		if vars.group.enabled then
			DCL:RegisterEvent('PARTY_INVITE_REQUEST')
			StaticPopupDialogs['PARTY_INVITE']=DCLD['PARTY_INVITE']
			StaticPopupDialogs['PARTY_INVITE_XREALM']=DCLD['PARTY_INVITE_XREALM']
		else
			DCL:UnregisterEvent('PARTY_INVITE_REQUEST')
			StaticPopupDialogs['PARTY_INVITE']=SPD_['PARTY_INVITE']
			StaticPopupDialogs['PARTY_INVITE_XREALM']=SPD_['PARTY_INVITE_XREALM']
		end
		if vars.chart.enabled then
			DCL:RegisterEvent('PETITION_SHOW')
			DCL_SetPetitionFrame('decline')
		else
			DCL:UnregisterEvent('PETITION_SHOW')
			DCL_SetPetitionFrame('default')
		end
		if vars.trade.enabled then
			DCL:RegisterEvent('TRADE_SHOW')
			DCL:RegisterEvent('TRADE_CLOSED')
			DCL_SetTradeFrame('decline')
		else
			DCL:UnregisterEvent('TRADE_SHOW')
			DCL:UnregisterEvent('TRADE_CLOSED')
			DCL_SetTradeFrame('default')
		end
		if vars.duel.enabled or vars.pet.enabled or vars.group.enabled then
			for i=1,STATICPOPUP_NUMDIALOGS do
				local frame=_G['StaticPopup'..i]
				DCL_SetStaticPopup(frame,'decline')
			end
		else
			for i=1,STATICPOPUP_NUMDIALOGS do
				local frame=_G['StaticPopup'..i]
				DCL_SetStaticPopup(frame,'default')
			end
		end
		if vars.duel.enabled or vars.pet.enabled or vars.trade.enabled then
			DCL:RegisterEvent('UI_INFO_MESSAGE')
		else
			DCL:UnregisterEvent('UI_INFO_MESSAGE')
		end
	end
end

-- disables the block trades and block guild invites options and edits their display text based on user options
-- Decliner won't be able to let authorized players through if these options are enabled

function DCL:BTBGI()
	if vars.guild.enabled and not vars.dcl.disable then
		_G['InterfaceOptionsControlsPanelBlockGuildInvitesText']:SetText('|cff666666'..BLOCK_GUILD_INVITES..' ('..a..')')
		SetAutoDeclineGuildInvites(false)
	else
		_G['InterfaceOptionsControlsPanelBlockGuildInvitesText']:SetText(BLOCK_GUILD_INVITES)
	end
	if vars.trade.enabled and not vars.dcl.disable then
		_G['InterfaceOptionsControlsPanelBlockTradesText']:SetText('|cff666666'..BLOCK_TRADES..' ('..a..')')
		SetCVar('blockTrades','0')
	else
		_G['InterfaceOptionsControlsPanelBlockTradesText']:SetText(BLOCK_TRADES)
	end
end

-- begin Interface Options panel
-- I would like to thank FlexYourHead of the addon Work_Complete for a ton of comments in their code
-- It made understanding buttons, text, and the options frame a whole lot easier

function DCL:CreateText(xloc,yloc,justify,text)
	numtxt=numtxt+1
	local name=a..'TextGlobal'..format('%02i',numtxt)
	local font=DCLP:CreateFontString(name,'ARTWORK','GameFontNormalSmall')
	font:SetPoint(justify,DCLP,'TOPLEFT',xloc,yloc)
	font:SetText(text)
	return font
end

function DCL:CreateBox(xloc,yloc,tooltip,text)
	numbox=numbox+1
	local name=a..'CheckBoxGlobal'..format('%02i',numbox)
	local box=CreateFrame('CheckButton',name,DCLP,'InterfaceOptionsCheckButtonTemplate')
	box:SetPoint('CENTER',DCLP,'TOPLEFT',xloc,yloc)
	box:SetHitRectInsets(0,0,0,0)
	box.tooltipText=tooltip
	_G[name..'Text']:SetText('|cffffd100'..text)
	return box
end

function DCL:Options()
	DCLP.name=a
	InterfaceOptions_AddCategory(DCLP,a)
	DCLtext01=DCL:CreateText(144,-32,'CENTER',S['DUE'])
	DCLtext02=DCL:CreateText(144,-48,'CENTER',S['REQ'])
	DCLtext03=DCL:CreateText(208,-32,'CENTER',S['PBA'])
	DCLtext04=DCL:CreateText(208,-48,'CENTER',S['REQ'])
	DCLtext05=DCL:CreateText(272,-32,'CENTER',S['GLD'])
	DCLtext06=DCL:CreateText(272,-48,'CENTER',S['INV'])
	DCLtext07=DCL:CreateText(336,-32,'CENTER',S['GRO'])
	DCLtext08=DCL:CreateText(336,-48,'CENTER',S['INV'])
	DCLtext09=DCL:CreateText(400,-32,'CENTER',S['GLD'])
	DCLtext10=DCL:CreateText(400,-48,'CENTER',S['PTN'])
	DCLtext11=DCL:CreateText(464,-32,'CENTER',S['TRA'])
	DCLtext12=DCL:CreateText(464,-48,'CENTER',S['REQ'])
	DCLtext13=DCL:CreateText(8,-80,'LEFT',S['EDE'])
	DCLbox01=DCL:CreateBox(144,-80,g..L['ENABLE_DUEL'],'')
	DCLbox02=DCL:CreateBox(208,-80,g..L['ENABLE_PET'],'')
	DCLbox03=DCL:CreateBox(272,-80,g..L['ENABLE_GUILD'],'')
	DCLbox04=DCL:CreateBox(336,-80,g..L['ENABLE_GROUP'],'')
	DCLbox05=DCL:CreateBox(400,-80,g..L['ENABLE_CHART'],'')
	DCLbox06=DCL:CreateBox(464,-80,g..L['ENABLE_TRADE'],'')
	DCLtext14=DCL:CreateText(8,-144,'LEFT',S['WHI'])
	DCLbox07=DCL:CreateBox(144,-144,r..L['TELL_DUEL'],'')
	DCLbox08=DCL:CreateBox(208,-144,r..L['TELL_PET'],'')
	DCLbox09=DCL:CreateBox(272,-144,r..L['TELL_GUILD'],'')
	DCLbox10=DCL:CreateBox(336,-144,r..L['TELL_GROUP'],'')
	DCLbox11=DCL:CreateBox(400,-144,r..L['TELL_CHART'],'')
	DCLbox12=DCL:CreateBox(464,-144,r..L['TELL_TRADE'],'')
	DCLtext15=DCL:CreateText(8,-208,'LEFT',S['GME'])
	DCLbox13=DCL:CreateBox(144,-208,r..L['GUILD_DUEL'],'')
	DCLbox14=DCL:CreateBox(208,-208,r..L['GUILD_PET'],'')
	DCLbox15=DCL:CreateBox(272,-208,r..L['GUILD_GUILD'],'')
	DCLbox16=DCL:CreateBox(336,-208,r..L['GUILD_GROUP'],'')
	DCLbox17=DCL:CreateBox(400,-208,r..L['GUILD_CHART'],'')
	DCLbox18=DCL:CreateBox(464,-208,r..L['GUILD_TRADE'],'')
	DCLtext16=DCL:CreateText(8,-272,'LEFT',S['RFR'])
	DCLbox19=DCL:CreateBox(144,-272,r..L['FRIEND_DUEL'],'')
	DCLbox20=DCL:CreateBox(208,-272,r..L['FRIEND_PET'],'')
	DCLbox21=DCL:CreateBox(272,-272,r..L['FRIEND_GUILD'],'')
	DCLbox22=DCL:CreateBox(336,-272,r..L['FRIEND_GROUP'],'')
	DCLbox23=DCL:CreateBox(400,-272,r..L['FRIEND_CHART'],'')
	DCLbox24=DCL:CreateBox(464,-272,r..L['FRIEND_TRADE'],'')
	DCLtext17=DCL:CreateText(8,-336,'LEFT',S['BNF'])
	DCLbox25=DCL:CreateBox(144,-336,r..L['BNET_DUEL'],'')
	DCLbox26=DCL:CreateBox(208,-336,r..L['BNET_PET'],'')
	DCLbox27=DCL:CreateBox(272,-336,r..L['BNET_GUILD'],'')
	DCLbox28=DCL:CreateBox(336,-336,r..L['BNET_GROUP'],'')
	DCLbox29=DCL:CreateBox(400,-336,r..L['BNET_CHART'],'')
	DCLbox30=DCL:CreateBox(464,-336,r..L['BNET_TRADE'],'')
	DCLtext18=DCL:CreateText(8,-400,'LEFT',S['SME'])
	DCLbox31=DCL:CreateBox(144,-400,r..S['HID']..' "'..a..':'..S['DUE']..'" '..S['SME'],'')
	DCLbox32=DCL:CreateBox(208,-400,r..S['HID']..' "'..a..':'..S['PBA']..'" '..S['SME'],'')
	DCLbox33=DCL:CreateBox(272,-400,r..S['HID']..' "'..a..':'..S['GLD']..'" '..S['SME'],'')
	DCLbox34=DCL:CreateBox(336,-400,r..S['HID']..' "'..a..':'..S['GRO']..'" '..S['SME'],'')
	DCLbox35=DCL:CreateBox(400,-400,r..S['HID']..' "'..a..':'..S['PTN']..'" '..S['SME'],'')
	DCLbox36=DCL:CreateBox(464,-400,r..S['HID']..' "'..a..':'..S['TRA']..'" '..S['SME'],'')
	DCLbox37=DCL:CreateBox(144,-464,r..L['X_DISABLE_MESSAGES'],S['DDM'])
	DCLbox38=DCL:CreateBox(144,-496,r..L['X_DISABLE_DECLINER'],S['DDE'])
	DCLbox01:SetScript('PostClick',function()vars.duel.enabled=DCLbox01:GetChecked()DCLU:Print()end)
	DCLbox02:SetScript('PostClick',function()vars.pet.enabled=DCLbox02:GetChecked()DCLU:Print()end)
	DCLbox03:SetScript('PostClick',function()vars.guild.enabled=DCLbox03:GetChecked()DCLU:Print()DCL:BTBGI()end)
	DCLbox04:SetScript('PostClick',function()vars.group.enabled=DCLbox04:GetChecked()DCLU:Print()end)
	DCLbox05:SetScript('PostClick',function()vars.chart.enabled=DCLbox05:GetChecked()DCLU:Print()end)
	DCLbox06:SetScript('PostClick',function()vars.trade.enabled=DCLbox06:GetChecked()DCLU:Print()DCL:BTBGI()end)
	DCLbox07:SetScript('PostClick',function()vars.duel.tell=DCLbox07:GetChecked()DCLU:Print()end)
	DCLbox08:SetScript('PostClick',function()vars.pet.tell=DCLbox08:GetChecked()DCLU:Print()end)
	DCLbox09:SetScript('PostClick',function()vars.guild.tell=DCLbox09:GetChecked()DCLU:Print()end)
	DCLbox10:SetScript('PostClick',function()vars.group.tell=DCLbox10:GetChecked()DCLU:Print()end)
	DCLbox11:SetScript('PostClick',function()vars.chart.tell=DCLbox11:GetChecked()DCLU:Print()end)
	DCLbox12:SetScript('PostClick',function()vars.trade.tell=DCLbox12:GetChecked()DCLU:Print()end)
	DCLbox13:SetScript('PostClick',function()vars.duel.guild=DCLbox13:GetChecked()DCLU:Print()end)
	DCLbox14:SetScript('PostClick',function()vars.pet.guild=DCLbox14:GetChecked()DCLU:Print()end)
	DCLbox15:SetScript('PostClick',function()if soundhandle then StopSound(soundhandle)end local x=random(1,51000)_,soundhandle=PlaySoundKitID(x)DCLbox15:SetChecked(false)end)
	DCLbox16:SetScript('PostClick',function()vars.group.guild=DCLbox16:GetChecked()DCLU:Print()end)
	DCLbox17:SetScript('PostClick',function()vars.chart.guild=DCLbox17:GetChecked()DCLU:Print()end)
	DCLbox18:SetScript('PostClick',function()vars.trade.guild=DCLbox18:GetChecked()DCLU:Print()end)
	DCLbox19:SetScript('PostClick',function()vars.duel.friend=DCLbox19:GetChecked()DCLU:Print()end)
	DCLbox20:SetScript('PostClick',function()vars.pet.friend=DCLbox20:GetChecked()DCLU:Print()end)
	DCLbox21:SetScript('PostClick',function()vars.guild.friend=DCLbox21:GetChecked()DCLU:Print()end)
	DCLbox22:SetScript('PostClick',function()vars.group.friend=DCLbox22:GetChecked()DCLU:Print()end)
	DCLbox23:SetScript('PostClick',function()vars.chart.friend=DCLbox23:GetChecked()DCLU:Print()end)
	DCLbox24:SetScript('PostClick',function()vars.trade.friend=DCLbox24:GetChecked()DCLU:Print()end)
	DCLbox25:SetScript('PostClick',function()vars.duel.bnet=DCLbox25:GetChecked()DCLU:Print()end)
	DCLbox26:SetScript('PostClick',function()vars.pet.bnet=DCLbox26:GetChecked()DCLU:Print()end)
	DCLbox27:SetScript('PostClick',function()vars.guild.bnet=DCLbox27:GetChecked()DCLU:Print()end)
	DCLbox28:SetScript('PostClick',function()vars.group.bnet=DCLbox28:GetChecked()DCLU:Print()end)
	DCLbox29:SetScript('PostClick',function()vars.chart.bnet=DCLbox29:GetChecked()DCLU:Print()end)
	DCLbox30:SetScript('PostClick',function()vars.trade.bnet=DCLbox30:GetChecked()DCLU:Print()end)
	DCLbox31:SetScript('PostClick',function()vars.duel.msg=DCLbox31:GetChecked()DCLU:Print()end)
	DCLbox32:SetScript('PostClick',function()vars.pet.msg=DCLbox32:GetChecked()DCLU:Print()end)
	DCLbox33:SetScript('PostClick',function()vars.guild.msg=DCLbox33:GetChecked()DCLU:Print()end)
	DCLbox34:SetScript('PostClick',function()vars.group.msg=DCLbox34:GetChecked()DCLU:Print()end)
	DCLbox35:SetScript('PostClick',function()vars.chart.msg=DCLbox35:GetChecked()DCLU:Print()end)
	DCLbox36:SetScript('PostClick',function()vars.trade.msg=DCLbox36:GetChecked()DCLU:Print()end)
	DCLbox37:SetScript('PostClick',function()vars.dcl.msgs=DCLbox37:GetChecked()DCLU:Print()end)
	DCLbox38:SetScript('PostClick',function()vars.dcl.disable=DCLbox38:GetChecked()DCLU:Print()DCL:BTBGI()end)
	DCLU:SetPoint('BOTTOMRIGHT',InterfaceOptionsFrame,-64,64)
	DCLU:SetHeight(8)
	DCLU:SetWidth(8)
	DCLU.text=DCLU:CreateFontString('DCLOptionUpdateAlert','ARTWORK','GameFontNormal')
	DCLU.text:SetPoint('RIGHT',DCLU)
	DCLP.okay=function()
		Decliner_SV=tcopy(vars)
		notemp=true
	end
	DCLP.cancel=function()
		vars=tcopy(temp)
		DCL:EventRegister()
		DCLU:Print('C')
		notemp=true
	end
	DCLP.default=function()
		vars=tcopy(defs)
		Decliner_SV=tcopy(defs)
		DCL:SetCheckBoxes()
		DCLU:Print()
		DCL:BTBGI()
	end
	DCLP.refresh=function()
		DCL:SetCheckBoxes()
		DCL:BTBGI()
		if notemp then temp=tcopy(vars) notemp=false end
	end
	SLASH_DECLINER_1='/decliner'
	SLASH_DECLINER_2='/decline'
	SLASH_DECLINER_3='/dcl'
	SlashCmdList['DECLINER_']=function()InterfaceOptionsFrame_OpenToCategory(a)end
end

function DCL:SetCheckBoxes()
	DCLbox01:SetChecked(vars.duel.enabled)
	DCLbox02:SetChecked(vars.pet.enabled)
	DCLbox03:SetChecked(vars.guild.enabled)
	DCLbox04:SetChecked(vars.group.enabled)
	DCLbox05:SetChecked(vars.chart.enabled)
	DCLbox06:SetChecked(vars.trade.enabled)
	DCLbox07:SetChecked(vars.duel.tell)
	DCLbox08:SetChecked(vars.pet.tell)
	DCLbox09:SetChecked(vars.guild.tell)
	DCLbox10:SetChecked(vars.group.tell)
	DCLbox11:SetChecked(vars.chart.tell)
	DCLbox12:SetChecked(vars.trade.tell)
	DCLbox13:SetChecked(vars.duel.guild)
	DCLbox14:SetChecked(vars.pet.guild)
	DCLbox15:SetChecked(false)
	DCLbox16:SetChecked(vars.group.guild)
	DCLbox17:SetChecked(vars.chart.guild)
	DCLbox18:SetChecked(vars.trade.guild)
	DCLbox19:SetChecked(vars.duel.friend)
	DCLbox20:SetChecked(vars.pet.friend)
	DCLbox21:SetChecked(vars.guild.friend)
	DCLbox22:SetChecked(vars.group.friend)
	DCLbox23:SetChecked(vars.chart.friend)
	DCLbox24:SetChecked(vars.trade.friend)
	DCLbox25:SetChecked(vars.duel.bnet)
	DCLbox26:SetChecked(vars.pet.bnet)
	DCLbox27:SetChecked(vars.guild.bnet)
	DCLbox28:SetChecked(vars.group.bnet)
	DCLbox29:SetChecked(vars.chart.bnet)
	DCLbox30:SetChecked(vars.trade.bnet)
	DCLbox31:SetChecked(vars.duel.msg)
	DCLbox32:SetChecked(vars.pet.msg)
	DCLbox33:SetChecked(vars.guild.msg)
	DCLbox34:SetChecked(vars.group.msg)
	DCLbox35:SetChecked(vars.chart.msg)
	DCLbox36:SetChecked(vars.trade.msg)
	DCLbox37:SetChecked(vars.dcl.msgs)
	DCLbox38:SetChecked(vars.dcl.disable)
end

local DCLU_OnUpdate=function()
	if (update < GetTime()-1) then
		local alpha=DCLU:GetAlpha()
		if (alpha ~= 0) then DCLU:SetAlpha(alpha-.05) end
		if (alpha == 0) then DCLU:SetScript('OnUpdate',nil) end
	end
end

function DCLU:Print(msg)
	DCL:EventRegister()
	if msg == 'C' then msg=r..S['CAN']
	else msg=g..S['OKA'] end
	DCLU.text:SetText(msg)
	DCLU:SetAlpha(1)
	update=GetTime()
	DCLU:SetScript('OnUpdate',DCLU_OnUpdate)
end

-- end Interface Options panel